proc runprg {w wlog proj prg wentry_fdat wentry_fres wentry_batch} {
	global env
	global input_type output_type run_mode priority
	global Glob_Pref Obj_Dir Obj_Proj

	set TkfPW_home $env(TKFPW_HOME)

	set projpath	$Obj_Dir(Proj)/$proj
	set Pgm		$projpath/Pgm
	set Builder	$projpath/Builder

	cd $projpath

	# Show Infos
	# ==========

 	SetWlogTargs $wlog

	$wlog delete 1.0 end

	$wlog insert end "Project\t:"
	$wlog insert end $proj BUTTON
	$wlog insert end \n

	$wlog insert end "Program\t:"
	$wlog insert end $prg BUTTON
	$wlog insert end \n

	$wlog insert end "Priority\t:"
	$wlog insert end $priority($w) OK
	$wlog insert end \n

	$wlog insert end "Run Mode\t:"
	$wlog insert end $run_mode($w) OK
	$wlog insert end \n

	$wlog insert end "Input Type\t:"
	$wlog insert end $input_type($w) OK
	$wlog insert end \n

	$wlog insert end "Output Type\t:"
	$wlog insert end $output_type($w) OK
	$wlog insert end \n

	# Build/Check parameters for the Run command
	# ==========================================

	set PRG	$Pgm/$prg

	switch -glob -- $priority($w) {
		{*high*}	{ set PRIORITY {} }
		{*low*}		{ set PRIORITY nice }
	}

	switch -glob -- $input_type($w) {
		{*no*}	{ 	set FDAT {}
				$wlog insert end \n
				$wlog insert end "No Input File !"
 			}
		default	{	set FDAT "[$wentry_fdat get]"
				$wlog insert end \n
				$wlog insert end "Input File : $FDAT"
				if ![ file exists $FDAT ] {
        				tk_dialog $w.d {TkfPW Error} "Unknown Input File\n$FDAT" error 0 {Try Again !}
					$wlog insert end \n
					$wlog insert end "Unknown Input File !" ERROR
					return
				}

 			}
	}

	switch -glob -- $output_type($w) {
		{*no*}	{ 	set FRES {}
				$wlog insert end \n
				$wlog insert end "No Output File !"
 			}
		default	{	set FRES "[$wentry_fres get]"
				$wlog insert end \n
				$wlog insert end "Output File : $FRES"
 			}
	}

	switch -glob -- $run_mode($w) {
		{*now*}	{	set DATE {}
				$wlog insert end \n
				$wlog insert end "Run Now"
			}
		{*bat*} {	set DATE "[$wentry_batch get]"
				$wlog insert end \n
				$wlog insert end "Run at : $DATE"
			}
	}

	# Generate the Run Command
	# ========================

	# Set Run Mode
	if { $PRIORITY == {} } {
		set MODE time
	} else {
		set MODE "$PRIORITY time"
	}	
	if { $FDAT == {} } {
		if { $FRES == {} } {
			set runcmd "$MODE $PRG"
		} else {
			set runcmd "$MODE $PRG >$FRES"
		}
	} else {
		if { $FRES == {} } {
			set runcmd "$MODE $PRG <$FDAT"
		} else {
			set runcmd "$MODE $PRG <$FDAT >$FRES"
		}
	}

	# Check for output file if batch mode (default is /dev/null)
	if { $DATE != {} } {
		if { $FRES == {} } {
			set FRES /dev/null
			$wlog insert end \n
			$wlog insert end "Warning : Results are sent to $FRES" ERROR
			set runcmd "$MODE $PRG <$FDAT >$FRES"
		}
	}

	# Generate an Executable Shell Macro
	# ==================================

	set macro $Builder/$prg.macro
	if [catch {open $macro w} fileId] {
		PRINT $fileId
	} else {
		puts $fileId "#!/bin/sh"
		puts $fileId "# Macro generated by TkfPW"
		puts $fileId "$runcmd"
		puts $fileId "echo 'Press Enter ...'"
		puts $fileId "read done"
		puts $fileId "exit"
		close $fileId
		exec chmod 755 $macro
	}

	# Run Macro
	# =========

	set XtermOpt_def $Glob_Pref(XtermOpt)
	set XtermOpt_imp {-T "TkfPW : Running Program $prg / Project $proj" -n "$prg" -sb}
	set Xterm $Glob_Pref(Xterm)

	if { $DATE != {} } {

		# Run Macro in Batch Mode

		if { $FRES == {} } {
			set FRES /dev/null
			$wlog insert end \n
			$wlog insert end "Warning : Results are sent to $FRES" ERROR
		}
			set ShellExecBatch $env(TKFPW_HOME)/cmd/SubmitBatch.sh
			if [ catch { eval exec $Xterm $XtermOpt_def $XtermOpt_imp -e $ShellExecBatch $macro [list $DATE] } err ] {
				tk_dialog $wlog.d {TkfPW Error} \
					"Batch Mode Failed !\Reason: $err" error 0 {Ooops !}
				$wlog insert end \n
				$wlog insert end "?? Batch Mode Failed" ERROR
				$wlog insert end \n
			} else {
				$wlog insert end \n
				$wlog insert end "Ok, Program is Launched" OK
				$wlog insert end \n
			}

	} else {

		# Run Macro in Xterm

		if [ catch { eval exec $Xterm $XtermOpt_def $XtermOpt_imp -e $macro & } XtermPid ] {
			tk_dialog $wlog.d {TkfPW Error} \
				"I Detected a problem on the X Window\
				Emulator :\n $XtermPid" error 0 {Ooops !}
			$wlog insert end \n
			$wlog insert end "?? Submit Failed" ERROR
			$wlog insert end \n
		} else {
# Ok, Xterm is running, register the process
#			set trace_type proj
#			set trace_id $numproj
#			set Process($trace_type,xterm,$trace_id,status) running
#			set Process($trace_type,xterm,$trace_id,pid) $XtermPid
			$wlog insert end \n
			$wlog insert end "Ok, Program is Launched" OK
			$wlog insert end \n
		}

	}
	
}
